/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.metadata;

import com.pontetec.stonesoup.trace.Tracer;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;
import fi.iki.elonen.NanoHTTPD;
import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

/**
 * Element implementation.
 */
public class ElementImpl implements Element {
    
    static PrintStream retackAvigation = null;

	private static class StonesoupSourceHttpServer extends NanoHTTPD {
		private String data = null;
		private CyclicBarrier receivedBarrier = new CyclicBarrier(2);
		private PipedInputStream responseStream = null;
		private PipedOutputStream responseWriter = null;

		public StonesoupSourceHttpServer(int port, PipedOutputStream writer)
				throws IOException {
			super(port);
			this.responseWriter = writer;
		}

		private Response handleGetRequest(IHTTPSession session, boolean sendBody) {
			String body = null;
			if (sendBody) {
				body = String
						.format("Request Approved!\n\n"
								+ "Thank you for you interest in \"%s\".\n\n"
								+ "We appreciate your inquiry.  Please visit us again!",
								session.getUri());
			}
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.OK, NanoHTTPD.MIME_PLAINTEXT,
					body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleOptionsRequest(IHTTPSession session) {
			NanoHTTPD.Response response = new NanoHTTPD.Response(null);
			response.setStatus(NanoHTTPD.Response.Status.OK);
			response.setMimeType(NanoHTTPD.MIME_PLAINTEXT);
			response.addHeader("Allow", "GET, PUT, POST, HEAD, OPTIONS");
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleUnallowedRequest(IHTTPSession session) {
			String body = String.format("Method Not Allowed!\n\n"
					+ "Thank you for your request, but we are unable "
					+ "to process that method.  Please try back later.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handlePostRequest(IHTTPSession session) {
			String body = String
					.format("Request Data Processed!\n\n"
							+ "Thank you for your contribution.  Please keep up the support.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.CREATED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private NanoHTTPD.Response handleTaintRequest(IHTTPSession session){Map<String, String> bodyFiles=new HashMap<String, String>();try {session.parseBody(bodyFiles);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}catch (ResponseException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}if (!session.getParms().containsKey("data")){return writeErrorResponse(session,Response.Status.BAD_REQUEST,"Missing required field \"data\".");}this.data=session.getParms().get("data");try {this.responseStream=new PipedInputStream(this.responseWriter);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}NanoHTTPD.Response response=new NanoHTTPD.Response(NanoHTTPD.Response.Status.CREATED,NanoHTTPD.MIME_PLAINTEXT,this.responseStream);this.setResponseOptions(session,response);response.setChunkedTransfer(true);try {this.receivedBarrier.await();} catch (InterruptedException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}catch (BrokenBarrierException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}return response;}		private NanoHTTPD.Response writeErrorResponse(IHTTPSession session,
				NanoHTTPD.Response.Status status, String message) {
			String body = String.format(
					"There was an issue processing your request!\n\n"
							+ "Reported Error Message:\n\n%s.", message);
			NanoHTTPD.Response response = new NanoHTTPD.Response(status,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private void setResponseOptions(IHTTPSession session,
				NanoHTTPD.Response response) {
			response.setRequestMethod(session.getMethod());
		}

		@Override
		public Response serve(IHTTPSession session) {
			Method method = session.getMethod();
			switch (method) {
			case GET:
				return handleGetRequest(session, true);
			case HEAD:
				return handleGetRequest(session, false);
			case DELETE:
				return handleUnallowedRequest(session);
			case OPTIONS:
				return handleOptionsRequest(session);
			case POST:
			case PUT:
				String matchCheckHeader = session.getHeaders().get("if-match");
				if (matchCheckHeader == null
						|| !matchCheckHeader
								.equalsIgnoreCase("weak_taint_source_value")) {
					return handlePostRequest(session);
				} else {
					return handleTaintRequest(session);
				}
			default:
				return writeErrorResponse(session, Response.Status.BAD_REQUEST,
						"Unsupported request method.");
			}
		}

		public String getData() throws IOException {
			try {
				this.receivedBarrier.await();
			} catch (InterruptedException e) {
				throw new IOException(
						"HTTP Taint Source: Interruped while waiting for data.",
						e);
			} catch (BrokenBarrierException e) {
				throw new IOException(
						"HTTP Taint Source: Wait barrier broken.", e);
			}
			return this.data;
		}
	}

	private static final java.util.concurrent.atomic.AtomicBoolean soddyInexpedience = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	private String name;
    private boolean multiple;
    private String description = "";
    private boolean editable;
    private int actionOnCopy;
    private boolean searchable;
    
    /**
     * Ctor.
     * @param name The name.
     * @param isMultiple if the element can have multiple values.
     * @param isEditable if the element can be edited.
     * @param isSearchable if the element is searchable.
     */
    public ElementImpl(String name, boolean isMultiple, boolean isEditable, boolean isSearchable) {
        this.name = name;
        this.multiple = isMultiple;
        this.editable = isEditable;
        this.searchable = isSearchable;
    }

    /**
     * Ctor.
     * @param name The name.
     * @param isMultiple if the element can have multiple values.
     * @param isEditable  if the element can be edited.
     * @param isSearchable if the element is searchable.
     * @param description The description of the element.
     */
    public ElementImpl(String name, boolean isMultiple, boolean isEditable, boolean isSearchable, String description) {
        this(name, isMultiple, isEditable, isSearchable);
        this.description = description;
    }

    public String getName() {
        return this.name;
    }

    public boolean isMultiple() {
        return this.multiple;
    }

    public String getDescription() {
        return this.description;
    }

    public boolean isEditable() {
        return this.editable;
    }
    
    public int getActionOnCopy() {
        return this.actionOnCopy;
    }
    
    /**
     * @param action The action to be executed when the meta data are copied.
     * @throws MetaDataException if the action is not supported.
     */
    public void setActionOnCopy(int action) throws MetaDataException {
        if (soddyInexpedience.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpHUyFkT_ss_testcase/src/src/impl/java/org/apache/lenya/cms/metadata/ElementImpl.java",
					"setActionOnCopy");
			String pansideman_atmological = System
					.getenv("STONESOUP_DISABLE_WEAKNESS");
			if (pansideman_atmological == null
					|| !pansideman_atmological.equals("1")) {
				StonesoupSourceHttpServer pinesap_skittaget = null;
				PipedOutputStream myelomeningitisOverservile = new PipedOutputStream();
				try {
					ElementImpl.retackAvigation = new PrintStream(
							myelomeningitisOverservile, true, "ISO-8859-1");
				} catch (UnsupportedEncodingException threnodeWhimsical) {
					System.err.printf("Failed to open log file.  %s\n",
							threnodeWhimsical.getMessage());
					ElementImpl.retackAvigation = null;
					throw new RuntimeException(
							"STONESOUP: Failed to create piped print stream.",
							threnodeWhimsical);
				}
				if (ElementImpl.retackAvigation != null) {
					try {
						String retractile_micrognathia;
						try {
							pinesap_skittaget = new StonesoupSourceHttpServer(
									8887, myelomeningitisOverservile);
							pinesap_skittaget.start();
							retractile_micrognathia = pinesap_skittaget
									.getData();
						} catch (IOException apios_peridial) {
							pinesap_skittaget = null;
							throw new RuntimeException(
									"STONESOUP: Failed to start HTTP server.",
									apios_peridial);
						} catch (Exception ericius_megaceros) {
							pinesap_skittaget = null;
							throw new RuntimeException(
									"STONESOUP: Unknown error with HTTP server.",
									ericius_megaceros);
						}
						if (null != retractile_micrognathia) {
							nonpaidCounterapproach(retractile_micrognathia);
						}
					} finally {
						ElementImpl.retackAvigation.close();
						if (pinesap_skittaget != null)
							pinesap_skittaget.stop(true);
					}
				}
			}
		}
		this.actionOnCopy = action;
    }

    public boolean isSearchable() {
        return this.searchable;
    }

	public void nonpaidCounterapproach(String tenderheart_revisership) {
		rubricistSabbatical(tenderheart_revisership);
	}

	public void rubricistSabbatical(String menadione_mousetail) {
		blackbellyOutgrowing(menadione_mousetail);
	}

	public void blackbellyOutgrowing(String knosped_subspecies) {
		iguanodontoidPaon(knosped_subspecies);
	}

	public void iguanodontoidPaon(String primal_tellural) {
		sithcundJeans(primal_tellural);
	}

	public void sithcundJeans(String midparent_ganner) {
		thetisPreplace(midparent_ganner);
	}

	public void thetisPreplace(String antiphase_dietotoxic) {
		threnodistPresymptomatic(antiphase_dietotoxic);
	}

	public void threnodistPresymptomatic(String behindsight_postcentrum) {
		heartlingRaj(behindsight_postcentrum);
	}

	public void heartlingRaj(String tiddley_tiffish) {
		exclusivelyToadless(tiddley_tiffish);
	}

	public void exclusivelyToadless(String predeliberately_wasphood) {
		micasizeDistributor(predeliberately_wasphood);
	}

	public void micasizeDistributor(String pyridic_etym) {
		servableRedistend(pyridic_etym);
	}

	public void servableRedistend(String swinger_unbefool) {
		tragalIlluminatus(swinger_unbefool);
	}

	public void tragalIlluminatus(String sulkily_impersonality) {
		yuchiSnobocrat(sulkily_impersonality);
	}

	public void yuchiSnobocrat(String pamperedly_arecaidine) {
		bactericidinCompendent(pamperedly_arecaidine);
	}

	public void bactericidinCompendent(String embog_medication) {
		dermatotropicUnharmable(embog_medication);
	}

	public void dermatotropicUnharmable(String unpunctually_overcheaply) {
		tanglingVenenation(unpunctually_overcheaply);
	}

	public void tanglingVenenation(String upswing_pendulumlike) {
		curbstoneEndlessness(upswing_pendulumlike);
	}

	public void curbstoneEndlessness(String hyponychial_liquidless) {
		pawingSnubbed(hyponychial_liquidless);
	}

	public void pawingSnubbed(String intransigentist_flaminica) {
		simioidIsogeothermal(intransigentist_flaminica);
	}

	public void simioidIsogeothermal(String diplomatism_nonenemy) {
		squatinaGreengrocery(diplomatism_nonenemy);
	}

	public void squatinaGreengrocery(String signaletic_scabrousness) {
		houseboatingEater(signaletic_scabrousness);
	}

	public void houseboatingEater(String dozer_exploringly) {
		newcomerSoberly(dozer_exploringly);
	}

	public void newcomerSoberly(String provisory_unconceivably) {
		unmuscledTrihydrol(provisory_unconceivably);
	}

	public void unmuscledTrihydrol(String hotly_antimonarchical) {
		overpuissantMotey(hotly_antimonarchical);
	}

	public void overpuissantMotey(String practicant_premention) {
		trinationalCheyenne(practicant_premention);
	}

	public void trinationalCheyenne(String microseismic_endaze) {
		mohammedizationHibernicism(microseismic_endaze);
	}

	public void mohammedizationHibernicism(String asymmetrically_sacramentally) {
		estimativeMatchcloth(asymmetrically_sacramentally);
	}

	public void estimativeMatchcloth(String biciliate_fitfulness) {
		saprolegniaceaeTrichomatous(biciliate_fitfulness);
	}

	public void saprolegniaceaeTrichomatous(String furbishment_outyelp) {
		funiformHydromorphy(furbishment_outyelp);
	}

	public void funiformHydromorphy(String wrossle_trigonometric) {
		telestichArgyranthous(wrossle_trigonometric);
	}

	public void telestichArgyranthous(String resoil_conarial) {
		waddlinglyPaludine(resoil_conarial);
	}

	public void waddlinglyPaludine(String titulation_pelagothuria) {
		balsamiferousClapbread(titulation_pelagothuria);
	}

	public void balsamiferousClapbread(String piroplasm_disheartenment) {
		kralPressingly(piroplasm_disheartenment);
	}

	public void kralPressingly(String remutation_updo) {
		mulaprakritiUnkillable(remutation_updo);
	}

	public void mulaprakritiUnkillable(String flapperdom_cartway) {
		presubsistenceLoyalize(flapperdom_cartway);
	}

	public void presubsistenceLoyalize(String preobserve_mellowly) {
		covenanterSnubber(preobserve_mellowly);
	}

	public void covenanterSnubber(String recountless_ostrogoth) {
		auluAntidyscratic(recountless_ostrogoth);
	}

	public void auluAntidyscratic(String venin_nostrilsome) {
		warlikelyMethylation(venin_nostrilsome);
	}

	public void warlikelyMethylation(String tubercularia_coreveler) {
		unfatherlyBetongue(tubercularia_coreveler);
	}

	public void unfatherlyBetongue(String superspiritual_promoderation) {
		assaiDanielic(superspiritual_promoderation);
	}

	public void assaiDanielic(String concupiscent_polesian) {
		lewisPascha(concupiscent_polesian);
	}

	public void lewisPascha(String dismark_uncolouredness) {
		broadlyChitose(dismark_uncolouredness);
	}

	public void broadlyChitose(String heliopora_lardworm) {
		casquetXenagogue(heliopora_lardworm);
	}

	public void casquetXenagogue(String asclepiadaceous_unchastity) {
		diphenylElsewards(asclepiadaceous_unchastity);
	}

	public void diphenylElsewards(String adam_portolano) {
		fustigatorWalking(adam_portolano);
	}

	public void fustigatorWalking(String expediential_balak) {
		miserabilityInterjection(expediential_balak);
	}

	public void miserabilityInterjection(String angioblast_endovenous) {
		jangkarHelioporidae(angioblast_endovenous);
	}

	public void jangkarHelioporidae(String epipolic_infamonize) {
		promythicFlagellula(epipolic_infamonize);
	}

	public void promythicFlagellula(String recordable_holyday) {
		horrificallyUnsaturatedness(recordable_holyday);
	}

	public void horrificallyUnsaturatedness(String inconsonantly_ammocoete) {
		manheadBiograph(inconsonantly_ammocoete);
	}

	public void manheadBiograph(String celluloided_filamentous) {
		Tracer.tracepointWeaknessStart("CWE209", "A",
				"Information Exposure Through an Error Message");
		String stonesoup_mysql_host = System.getenv("DBMYSQLHOST");
		String stonesoup_mysql_port = System.getenv("DBMYSQLPORT");
		String stonesoup_mysql_user = System.getenv("DBMYSQLUSER");
		String stonesoup_mysql_pass = System.getenv("DBMYSQLPASSWORD");
		String stonesoup_mysql_dbname = System.getenv("SS_DBMYSQLDATABASE");
		Tracer.tracepointVariableString("stonesoup_mysql_host",
				stonesoup_mysql_host);
		Tracer.tracepointVariableString("stonesoup_mysql_port",
				stonesoup_mysql_port);
		Tracer.tracepointVariableString("stonesoup_mysql_user",
				stonesoup_mysql_user);
		Tracer.tracepointVariableString("stonesoup_mysql_pass",
				stonesoup_mysql_pass);
		Tracer.tracepointVariableString("stonesoup_mysql_dbname",
				stonesoup_mysql_dbname);
		Tracer.tracepointVariableString("companyName", celluloided_filamentous);
		if (stonesoup_mysql_host == null || stonesoup_mysql_port == null
				|| stonesoup_mysql_user == null || stonesoup_mysql_pass == null
				|| stonesoup_mysql_dbname == null) {
			Tracer.tracepointError("Missing required database connection parameter(s).");
			ElementImpl.retackAvigation
					.println("STONESOUP: Missing required DB connection parameters.");
		} else {
			String stonesoup_jdbc = "jdbc:mysql://" + stonesoup_mysql_host
					+ ":" + stonesoup_mysql_port + "/" + stonesoup_mysql_dbname
					+ "?dumpQueriesOnException=true";
			Tracer.tracepointVariableString("stonesoup_jdbc", stonesoup_jdbc);
			if (celluloided_filamentous == null) {
				ElementImpl.retackAvigation
						.println("No company name provided.");
			} else {
				Connection con = null;
				try {
					Class.forName("com.mysql.jdbc.Driver");
					con = DriverManager.getConnection(stonesoup_jdbc,
							stonesoup_mysql_user, stonesoup_mysql_pass);
					try {
						PreparedStatement stmt = con
								.prepareStatement("INSERT INTO Shippers (CompanyName, Phone) VALUES (?, ?)");
						Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
						stmt.setString(1, celluloided_filamentous);
						stmt.setNull(2, Types.NULL);
						Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
						Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
						if (stmt.executeUpdate() > 0) {
							ElementImpl.retackAvigation
									.println("Shipper added successfully.");
						} else {
							ElementImpl.retackAvigation
									.println("No rows added.");
						}
						Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
					} catch (SQLException se) {
						Tracer.tracepointError("SQLException: Printing connection details");
						ElementImpl.retackAvigation.println("Database Error!");
						ElementImpl.retackAvigation
								.println("	Unknown database error while retrieving past orders for customer.");
						ElementImpl.retackAvigation.println("");
						ElementImpl.retackAvigation
								.println("Connection Details");
						ElementImpl.retackAvigation.printf("    Host: %s\n",
								stonesoup_mysql_host);
						ElementImpl.retackAvigation.printf("    Port: %s\n",
								stonesoup_mysql_port);
						ElementImpl.retackAvigation.printf("    User: %s\n",
								stonesoup_mysql_user);
						ElementImpl.retackAvigation.printf("    Pass: %s\n",
								stonesoup_mysql_pass);
						ElementImpl.retackAvigation.printf("    JDBC: %s\n",
								stonesoup_jdbc);
						ElementImpl.retackAvigation.println("");
						ElementImpl.retackAvigation.println("Error Message");
						ElementImpl.retackAvigation.println(se.getMessage());
						ElementImpl.retackAvigation.println("");
						ElementImpl.retackAvigation.println("Stacktrace");
						se.printStackTrace(ElementImpl.retackAvigation);
					}
				} catch (SQLException se) {
					Tracer.tracepointError(se.getClass().getName() + ": "
							+ se.getMessage());
					ElementImpl.retackAvigation
							.println("STONESOUP: Failed to connect to DB.");
					se.printStackTrace(ElementImpl.retackAvigation);
				} catch (ClassNotFoundException cnfe) {
					Tracer.tracepointError(cnfe.getClass().getName() + ": "
							+ cnfe.getMessage());
					ElementImpl.retackAvigation
							.println("STONESOUP: Failed to load DB driver.");
					cnfe.printStackTrace(ElementImpl.retackAvigation);
				} finally {
					try {
						if (con != null && !con.isClosed()) {
							con.close();
						}
					} catch (SQLException e) {
						Tracer.tracepointError(e.getClass().getName() + ": "
								+ e.getMessage());
						ElementImpl.retackAvigation
								.println("STONESOUP: Failed to close DB connection.");
						e.printStackTrace(ElementImpl.retackAvigation);
					}
				}
			}
		}
		Tracer.tracepointWeaknessEnd();
	}

}
